package com.fjhx.victoriatourist.service.purchase.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.fjhx.victoriatourist.entity.purchase.dto.PurchaseBackDto;
import com.fjhx.victoriatourist.entity.purchase.dto.PurchaseBackSelectDto;
import com.fjhx.victoriatourist.entity.purchase.po.PurchaseBack;
import com.fjhx.victoriatourist.entity.purchase.po.PurchaseBackDetails;
import com.fjhx.victoriatourist.entity.purchase.vo.PurchaseBackDetailsVo;
import com.fjhx.victoriatourist.entity.purchase.vo.PurchaseBackVo;
import com.fjhx.victoriatourist.mapper.purchase.PurchaseBackMapper;
import com.fjhx.victoriatourist.service.purchase.PurchaseBackDetailsService;
import com.fjhx.victoriatourist.service.purchase.PurchaseBackService;
import com.ruoyi.common.utils.wrapper.IWrapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;


/**
 * <p>
 * 采购退货 服务实现类
 * </p>
 *
 * @author
 * @since 2023-05-19
 */
@Service
public class PurchaseBackServiceImpl extends ServiceImpl<PurchaseBackMapper, PurchaseBack> implements PurchaseBackService {
    @Autowired
    PurchaseBackDetailsService purchaseBackDetailsService;

    @Override
    public Page<PurchaseBackVo> getPage(PurchaseBackSelectDto dto) {
        IWrapper<PurchaseBack> wrapper = getWrapper();
        wrapper.orderByDesc("pb", PurchaseBack::getId);
        Page<PurchaseBackVo> page = this.baseMapper.getPage(dto.getPage(), wrapper);
        return page;
    }

    @Override
    public PurchaseBackVo detail(Long id) {
        PurchaseBack PurchaseBack = this.getById(id);
        PurchaseBackVo result = BeanUtil.toBean(PurchaseBack, PurchaseBackVo.class);
        List<PurchaseBackDetails> purchaseBackDetails = purchaseBackDetailsService.list(q -> q.eq(PurchaseBackDetails::getPurchaseBackId, result.getId()));
        List<PurchaseBackDetailsVo> purchaseBackDetailsVos = BeanUtil.copyToList(purchaseBackDetails, PurchaseBackDetailsVo.class);
        result.setPurchaseBackDetailsList(purchaseBackDetailsVos);
        return result;
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void add(PurchaseBackDto purchaseBackDto) {
        this.save(purchaseBackDto);
        List<PurchaseBackDetails> purchaseBackDetailsList = purchaseBackDto.getPurchaseBackDetailsList();
        purchaseBackDetailsList.forEach(purchaseBackDetails -> purchaseBackDetails.setPurchaseBackId(purchaseBackDto.getId()));
        purchaseBackDetailsService.saveBatch(purchaseBackDetailsList);
    }

}
